/*-
 * Copyright (c) 2010 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Alistair Crooks (agc@NetBSD.org)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "s4.h"

int 
main(int argc, char **argv)
{
	s4strings_t	 res;
	s4str_t		 str;
	s4_t		 s4;
	char		*output;
	int		 reconstruct;
	int		 threshold;
	int		 sharesc;
	int		 ok;
	int		 i;

	(void) memset(&res, 0x0, sizeof(res));
	(void) memset(&str, 0x0, sizeof(str));
	reconstruct = 0;
	output = NULL;
	threshold = sharesc = 0;
	while ((i = getopt(argc, argv, "o:rs:t:")) != -1) {
		switch(i) {
		case 'o':
			output = optarg;
			break;
		case 'r':
			reconstruct = 1;
			break;
		case 's':
			sharesc = atoi(optarg);
			break;
		case 't':
			threshold = atoi(optarg);
			break;
		default:
			break;
		}
	}
	(void) s4_init(&s4);
	if (reconstruct) {
		if ((ok = s4_reconstruct_files(&s4, &argv[optind], argc - optind, argc - optind, &str)) != 0) {
			ok = s4_reconstruct_write(&str, output);
		}
	} else {
		if ((ok = s4_split_file(&s4, argv[optind], sharesc, threshold, &res)) != 0) {
			ok = s4_split_write(&res, argv[optind], sharesc);
		}
	}
	(void) s4_end(&s4);
	exit((ok) ? EXIT_SUCCESS : EXIT_FAILURE);
}
