/*-
 * Copyright (c) 2010 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Alistair Crooks (agc@NetBSD.org)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef SHARES_H_
#define SHARES_H_	20100130

#ifndef __BEGIN_DECLS
#  if defined(__cplusplus)
#  define __BEGIN_DECLS           extern "C" {
#  define __END_DECLS             }
#  else
#  define __BEGIN_DECLS
#  define __END_DECLS
#  endif
#endif

__BEGIN_DECLS

/* an s4 structure to hold local vars */
typedef struct s4_t {
	int		 sharesc;	/* # of shares */
	unsigned	 extra;
	unsigned	 magic;
	int		 have_extra;
	int		 oddflag;
	int		*seen;		/* if seen, then not the first */
	unsigned	*next_d1;
} s4_t;

/* an s4 string */
typedef struct s4str_t {
	unsigned	 size;		/* size of allocation */
	unsigned	 c;		/* # of chars */
	uint8_t		*v;		/* the string itself */
} s4str_t;

/* an s4 set of strings */
typedef struct s4strings_t {
	unsigned	 c;		/* # of strings */
	s4str_t		*str;		/* the strings themselves */
} s4strings_t;

int s4_init(s4_t *);
int s4_end(s4_t *);

int s4_split_file(s4_t *, char *, int, int, s4strings_t *);
int s4_split_memory(s4_t *, char *, unsigned, int, int, s4strings_t *);
int s4_split_write(s4strings_t *, const char *, int);

int s4_reconstruct_files(s4_t *, char **, int, int, s4str_t *);
int s4_reconstruct_memory(s4_t *s4, char **, unsigned, int, int, s4str_t *);
int s4_reconstruct_write(s4str_t *, const char *);

__END_DECLS

#endif
